@@ -1,7 +1,7 @@
 #! /usr/bin/env python3
 import logging
 logging.info("Importing packages in base_runner")
-from easy_module_attribute_getter import YamlReader, PytorchGetter
+from easy_module_attribute_getter import YamlReader, PytorchGetter, utils as emag_utils
 from ..utils import common_functions as c_f
 import argparse
 import glob
@@ -35,13 +35,17 @@ class BaseRunner:
     def register(self, module_type, module):
         self.pytorch_getter.register(module_type, module)
 
+    def unregister(self, module_type, module):
+        self.pytorch_getter.unregister(module_type, module)
+
     def init_pytorch_getter(self):
-        from pytorch_metric_learning import trainers, losses, miners, regularizers, samplers, testers, utils
-        from .. import architectures
-        from .. import datasets
-        from .. import api_parsers
+        from pytorch_metric_learning import trainers, distances, losses, miners, reducers, regularizers, samplers, testers, utils
+        from .. import architectures, datasets, factories, api_parsers, split_managers, aggregators, ensembles
+        from sklearn.manifold import TSNE
+        import torch
         self.pytorch_getter = PytorchGetter(use_pretrainedmodels_package=True)
         self.pytorch_getter.register('model', architectures.misc_models)
+        self.pytorch_getter.register('model', utils.common_functions.Identity)
         self.pytorch_getter.register('loss', losses)
         self.pytorch_getter.register('miner', miners)
         self.pytorch_getter.register('regularizer', regularizers)
@@ -50,7 +54,16 @@ class BaseRunner:
         self.pytorch_getter.register('tester', testers)
         self.pytorch_getter.register('dataset', datasets)
         self.pytorch_getter.register('api_parser', api_parsers)
-        self.pytorch_getter.register('accuracy_calculator', utils.AccuracyCalculator)
+        self.pytorch_getter.register('accuracy_calculator', utils.accuracy_calculator.AccuracyCalculator)
+        self.pytorch_getter.register('split_manager', split_managers)
+        self.pytorch_getter.register('hook_container', utils.logging_presets.HookContainer)
+        self.pytorch_getter.register('factory', factories)
+        self.pytorch_getter.register('aggregator', aggregators)
+        self.pytorch_getter.register('ensemble', ensembles)
+        self.pytorch_getter.register('visualizer', TSNE)
+        self.pytorch_getter.register('distance', distances)
+        self.pytorch_getter.register('reducer', reducers)
+        self.pytorch_getter.register('weight_init_func', torch.nn.init)
 
 
     def set_YR(self):
@@ -63,13 +76,25 @@ class BaseRunner:
 
     def get_api_parser(self, args):
         api_parser_kwargs = {"args": args, "pytorch_getter": self.pytorch_getter, "global_db_path": self.global_db_path}
-        return self.pytorch_getter.get('api_parser', class_name="API"+args.training_method, params=api_parser_kwargs)
+        if args.api_parser:
+            AP, AP_params = self.pytorch_getter.get('api_parser', yaml_dict=args.api_parser, return_uninitialized=True)
+            AP_params = emag_utils.merge_two_dicts(api_parser_kwargs, AP_params)
+            AP = AP(**AP_params)
+        else:
+            class_name = "API"+c_f.first_key_of_dict(args.trainer)
+            try:
+                AP = self.pytorch_getter.get('api_parser', class_name=class_name, params=api_parser_kwargs)
+            except AttributeError:
+                logging.warning("Couldn't find api_parser named {}. Using BaseAPIParser instead.".format(class_name))
+                AP = self.pytorch_getter.get('api_parser', class_name="BaseAPIParser", params=api_parser_kwargs)
+        return AP
 
     def setup_argparser(self):
         parser = argparse.ArgumentParser(allow_abbrev=False)
         parser.add_argument("--experiment_name", type=str, required=True)
         parser.add_argument("--resume_training", type=str, default=None, choices=["latest", "best"])
         parser.add_argument("--evaluate", action="store_true")
+        parser.add_argument("--evaluate_ensemble", action="store_true")
         parser.add_argument("--reproduce_results", type=str, default=None)
         return parser
 
@@ -99,7 +124,7 @@ class BaseRunner:
 
 
     def determine_where_to_get_yamls(self, args):
-        if args.resume_training or args.evaluate:
+        if args.resume_training or args.evaluate or args.evaluate_ensemble:
             config_paths = self.get_saved_config_paths(args)
         else:
             config_paths = self.get_root_config_paths(args)
